unit u_webbookscreate;

{$IFDEF FPC}
{$R *.lfm}
{$mode objfpc}{$H+}
{$ELSE}
{$DEFINE WINDOWS}
{$R *.lfm}
{$ENDIF}

////////////////////////////////////////////////////////////////////////////////
// Satic Web Books Create
// Création de sites web statiques de livrels libres et gratuits.
// Licence : GPL
// LIBERLOG 2019
// Auteur : Matthieu GIROUX
// Descriptions
// Création d'un arbre complet, d'une page de contact en PHP, de fiches, etc.
// Historique
// 0.9.9.0 : première version publiée
////////////////////////////////////////////////////////////////////////////////

interface

uses
{$ifdef unix}
  BaseUnix,
{$endif}

{$IFNDEF FPC}
  Mask,  rxToolEdit, JvExControls,
{$ELSE}
  lazutf8classes,
  LCLIntf, LCLType, EditBtn,FileUtil,
{$ENDIF}
  fonctions_string,
{$ifdef windows}
  Windows,
{$endif}
{$IFDEF WIN32}
  Registry,
{$ENDIF}
{$IFNDEF FPC}
  fonctions_version,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, DB,
  DBCtrls, ExtCtrls, Buttons, ComCtrls, DBGrids,
  functions_html, ExtJvXPCheckCtrls, Spin, JvXPCheckCtrls, U_OnFormInfoIni,
  u_buttons_appli, U_ExtFileCopy,
  IniFiles,
  fonctions_web,
  u_extabscopy,
  BGRABitmap,
  u_buttons_defs;

{$IFNDEF FPC}
const
  gVer_WebPlayer : T_Version = ( Component : 'Application WebBooksCreator' ;
                                             FileUnit : 'U_WebBooksCreate' ;
                                             Owner : 'Matthieu Giroux' ;
                                             Comment : 'Créateur de sites web HTNL de livels.' ;
                                             BugsStory :  '0.9.9.0 : First published version'  ;
                                             UnitType : CST_TYPE_UNITE_APPLI ;
                                             Major : 0 ; Minor : 9 ; Release : 9 ; Build : 0 );
{$ENDIF}

const CST_WebBooks = 'WebBooksCreator' ;
      CST_WebBooks_WithLicense = CST_WebBooks + ' GPL ';
      CST_EXTENSION_PDF_MINI = 'pdf';
      CST_EXTENSION_EPUB_MINI = 'epub';
      CST_EXTENSION_ODT_MINI = 'odt';
      CST_EXTENSION_CSV_MINI = 'csv';
      CST_EXTENSION_PDF = '.'+CST_EXTENSION_PDF_MINI;
      CST_EXTENSION_EPUB = '.'+CST_EXTENSION_EPUB_MINI;
      CST_EXTENSION_ODT  = '.'+CST_EXTENSION_ODT_MINI;
      CST_EXTENSION_CSV = '.'+CST_EXTENSION_CSV_MINI;
      CST_SCRIPT_FILE   = 'conversion';

type

  { TF_WebBooks }

  TF_WebBooks = class(TForm)
    bt_gen: TFWExport;
    bt_see: TFWSearch;
    cb_Files: TComboBox;
    cb_Themes: TComboBox;
    ch_EPUB1: TJvXPCheckbox;
    ch_Picture: TJvXPCheckbox;
    ch_GZ: TJvXPCheckbox;
    ch_downloads: TJvXPCheckbox;
    ch_convert: TJvXPCheckbox;
    ch_Names64: TJvXPCheckbox;
    ch_EPUB: TJvXPCheckbox;
    ch_Torrent: TJvXPCheckbox;
    ch_RecurseSubDirs: TJvXPCheckbox;
    ch_ZIP: TJvXPCheckbox;
    ch_CSV: TJvXPCheckbox;
    ch_7ZIP: TJvXPCheckbox;
    de_indexdir: TDirectoryEdit;
    de_websitedir: TDirectoryEdit;
    ed_Author: TEdit;
    ed_IndexName: TEdit;
    ed_DownLoadName: TEdit;
    ed_TXTName: TEdit;
    FileCopy: TExtFileCopy;
    ds_Individu: TDatasource;
    FileIniCopy: TExtFileCopy;
    fne_export: TFileNameEdit;
    fne_import: TFileNameEdit;
    FWClose1: TFWClose;
    bt_export: TFWExport;
    ch_PDF: TJvXPCheckbox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label44: TLabel;
    Label6: TLabel;
    lb_Comments: TLabel;
    lb_Images: TLabel;
    lb_Images1: TLabel;
    L_thema: TLabel;
    me_Bottom: TMemo;
    me_Description: TMemo;
    OnFormInfoIni: TOnFormInfoIni;
    OpenDialog: TOpenDialog;
    pa_options: TPanel;
    pb_Progress: TProgressBar;
    spSkinPanel1: TPanel;
    procedure bt_exportClick(Sender: TObject);
    procedure bt_genClick(Sender: TObject);
    procedure bt_seeClick(Sender: TObject);
    procedure fne_ExportAcceptFileName(Sender: TObject; var Value: String);
    procedure fne_importAcceptFileName(Sender: TObject; var Value: String);
    procedure fne_importChange(Sender: TObject);
  private
    { Déclarations privées }
    procedure DoAfterInit(const ab_Ini: Boolean=True);
    function fb_DeleteOrNot(const as_ThemaSource : String; astl_ListFiles: TStrings): Boolean;
    function fstl_CreateListToDelete ( const as_ThemaSource : String ): TStringListUTF8;
    procedure p_AddFiles ( const as_Source, as_artist, as_subdirForward : String ;
                           const astl_files : TStrings;
                           const ai_eraseBegin, ai_Level : Integer;
                           const astl_DirListAudio, astl_temp1,
                                 astl_HtmlDownloads, astl_downloadsafter , astl_Processes : TStrings;
                           var   ab_first, ab_foundAudio : Boolean;
                           const ab_Root : Boolean ; var ab_downloadReally, ab_TextOneLanguage : Boolean;
                           var   as_BackDirText : String);
    procedure p_CreateAHtmlFile(const astl_Destination: TStrings;
      const as_BeginingFile, as_Describe, as_file, as_Title, as_LittleTitle, as_LongTitle: string;
      const as_Subdir: string = '';
      const as_ExtFile: string = CST_EXTENSION_HTML_PLAIN;
      const as_BeforeHTML: string = ''; const astl_Body : TStrings = nil );
    procedure p_genHtmlHome ( const as_directory, as_subdirForward, as_artist, as_backDirText : String;
                              const ab_TextOneLanguage, ab_Root : Boolean );
    procedure p_comment(const as_Comment: String);
    procedure p_Unfat(const as_directory: String);
  public
    { Déclarations publiques }
    constructor Create(aowner:TComponent); override;
  end;

var
  F_WebBooks: TF_WebBooks;
  gs_sourcedir : String;
  gb_Generate : boolean = False;


implementation

uses  fonctions_ini,
  functions_html_tree,
{$IFDEF FPC}
  unite_messages,
  web_strings,
  webbooks_strings, strutils,
{$ELSE}
  unite_messages_delphi,
  web_strings_delphi,
  webbooks_strings_delphi,
{$ENDIF}
{$IFDEF WIN32}
//  windirs,
{$ENDIF}
  fonctions_system,
  LazFileUtils,
  fonctions_languages,
  fonctions_images,
  fonctions_filepath,
  fonctions_components,
  fonctions_file;

procedure TF_WebBooks.bt_genClick(Sender: TObject);
var lb_gen : Boolean;
    ls_question : String;
var ls_Source : String;
Begin
  if not fb_genUnGenPrepare(de_indexdir,cb_Files,cb_Themes,ls_Source)
   Then
    Exit;
  gs_html_source_file := CST_SUBDIR_FILES + DirectorySeparator + cb_files.Items[cb_files.ItemIndex]+DirectorySeparator;
  gs_RootPathForExport := IncludeTrailingPathDelimiter ( de_websitedir.Directory );
  gs_sourcedir         := IncludeTrailingPathDelimiter ( de_indexdir  .Directory );
  lb_gen := False;
  if not DirectoryExistsUTF8(gs_sourcedir) Then
   Begin
    MessageDlg(gs_Error_only,gs_error_WebSite_Sources_do_not_exists,mtError,[mbOK],0,mbOK);
    Exit;
   end;
  if not DirectoryExistsUTF8(gs_RootPathForExport) Then
   Begin
    if MessageDlg(gs_Warning,fs_RemplaceMsg(gs_confirm_WebSite_directory_do_not_exists_create_it,[gs_RootPathForExport]),mtConfirmation,mbYesNo,0,mbYes)=mrYes Then
     try
      fb_CreateDirectoryStructure(gs_RootPathForExport);
     finally
     end;
    if not DirectoryExistsUTF8(gs_RootPathForExport) Then
     Begin
       MessageDlg(gs_Error_only,gs_error_cannot_create_WebSite_directory,mtError,[mbOK],0,mbOK);
       Exit;
     end;
   end;
  if pos(de_websitedir.Directory,de_indexdir.Directory)>0 Then
   Begin
    MessageDlg(gs_Error_only,gs_cannot_create_website_including_sources,mtError,[mbOK],0,mbOK);
    Exit;
   end;
  ls_question := fs_RemplaceMsg ( gs_warning_WebSite_Create,[gs_RootPathForExport]);
  if FileExistsUTF8(gs_RootPathForExport+ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN) Then
   Begin
    if ch_convert.Checked Then
     AppendStr(ls_question,gs_warning_Converting_can_be_long+gs_warning_Every_Converted_Files_Rewritten);
    AppendStr(ls_question,gs_warning_Every_WebSite_Files_Rewritten+gs_ready);
    lb_gen:=MessageDlg(gs_Warning,ls_question,mtConfirmation,mbOKCancel,0)=mrOK;
   end
  Else
   if DirectoryIsEmpty(gs_RootPathForExport) Then
     Begin
       if ch_convert.Checked Then
         AppendStr(ls_question,gs_warning_Converting_can_be_long);
       AppendStr(ls_question,gs_ready);
       lb_gen:=MessageDlg(gs_Warning,ls_question,mtConfirmation,mbYesNo,0,mbYes)=mrYes;
     end;
  if lb_gen Then
    try // starting work
      p_CreateKeyWords;
      gb_Generate := True;
      p_genHtmlHome ( gs_RootPathForExport, '', '', '', False, True );
    finally
      pa_options.Enabled:=True;
      gb_Generate := False;
      p_comment ( gs_Web_Finished ); // advert for user
    end;
end;

procedure TF_WebBooks.bt_seeClick(Sender: TObject);
begin
  p_OpenFileOrDirectory(de_websitedir.Directory);
end;


// procedure TF_WebBooks.fne_ExportAcceptFileName
// Exporting ini on filename's accept
procedure TF_WebBooks.fne_ExportAcceptFileName(Sender: TObject;
  var Value: String);
begin
  bt_exportClick(bt_export);
end;

function TF_WebBooks.fb_DeleteOrNot(const as_ThemaSource: String;
  astl_ListFiles: TStrings): Boolean;
begin

end;

function TF_WebBooks.fstl_CreateListToDelete(const as_ThemaSource: String
  ): TStringListUTF8;
begin

end;

procedure TF_WebBooks.p_AddFiles ( const as_Source, as_artist, as_subdirForward : String ;
                                    const astl_files : TStrings;
                                    const ai_eraseBegin, ai_Level : Integer;
                                    const astl_DirListAudio, astl_temp1,
                                          astl_HtmlDownloads, astl_downloadsafter, astl_Processes : TStrings;
                                    var   ab_first, ab_foundAudio : Boolean;
                                    const ab_Root : Boolean ; var ab_downloadReally, ab_TextOneLanguage : Boolean;
                                    var  as_BackDirText : String );
var li_EndExt : Integer ;
    ls_FileSource ,
    ls_SourceConvert ,
    ls_DirHtml ,
    ls_FileName ,
    ls_FileNameLower ,
    ls_destination  : String;
    lsr_AttrSource      : Tsearchrec;
    lb_TextOneLanguage,
    lb_first : Boolean;
    li_pos : Integer;
    ls_BackDirText:String;
    lstl_temp2 : TStringListUTF8 ;

    // mini ls_source
    procedure p_MiniFileSource;
    Begin
      {$IFDEF WINDOWS}
      ls_FileSource := copy(fs_RemplaceChar(StringReplace(ls_FileSource,'"','\"',[rfReplaceAll]),DirectorySeparator,'/'),ai_eraseBegin,length(ls_FileSource)-ai_eraseBegin+1);
      {$ELSE}
      ls_FileSource := copy(StringReplace(ls_FileSource,'"','\"',[rfReplaceAll]),ai_eraseBegin,length(ls_FileSource)-ai_eraseBegin+1);
      {$ENDIF}

    end;

    // mini ls_source
    procedure p_MiniSource ( const as_FileSource : String );
    Begin
      ls_FileSource := copy(StringReplace(as_FileSource,'"','\"',[rfReplaceAll]),ai_eraseBegin,length(ls_FileSource)-ai_eraseBegin+1);
    end;

    // replace string for a line of music
    procedure p_replaceAllCase ( const ai_pos : Integer ; const as_Ext : String ; const ab_ishref : Boolean = True );
     Begin
       p_ReplaceLanguageString(astl_temp1,'Source',StringReplace(ls_FileSource,'"','\"',[rfReplaceAll]),[rfReplaceAll]);
       if ab_ishref
        Then p_ReplaceLanguageString(astl_temp1,'SourceTitle',copy(StringReplace(StringReplace(ls_FileName,'"','\"',[rfReplaceAll]),'_',' ',[rfReplaceAll]),1,ai_pos-1),[rfReplaceAll])
        Else p_ReplaceLanguageString(astl_temp1,'SourceTitle',copy(StringReplace(StringReplace(ls_FileName,'_',' ',[rfReplaceAll]),'"','\"',[rfReplaceAll]),1,ai_pos-1),[rfReplaceAll]);
       p_ReplaceLanguageString(astl_temp1,'Type','audio/'+as_Ext,[rfReplaceAll]);
     end;

    // Add a download to music download list
    procedure p_addDownload ( const astl_downloadList : TStrings ; const ai_pos : Integer ; const as_Ext : String ); overload;
    Begin
      p_LoadStringList ( astl_temp1,  CST_DOWNLOAD_FILE+CST_EXTENSION_HTML );
      p_replaceAllCase ( ai_pos, as_Ext );
      if as_artist > ''
       Then p_ReplaceLanguageString(astl_temp1,'SourceArtist',' - '+as_artist,[rfReplaceAll])
       Else p_ReplaceLanguageString(astl_temp1,'SourceArtist','',[rfReplaceAll]);
      astl_downloadList.AddStrings(astl_temp1);
    end;

    // Add a download to music download list
    procedure p_addExternalDownload ( const astl_downloadList : TStrings ; const as_Link : String ); overload;
    var astl_temp1 : TStringList;
        li_i ,
        li_length : Longint;
    Begin
      li_length := length ( as_Link );
      li_pos := PosEx ( '.', as_Link, li_length - 5 );
      while PosEx ( '.', as_Link, li_pos + 1 ) > 0 do
       li_pos := PosEx ( '.', as_Link, li_pos + 1 );
      if li_pos = 0 Then Exit;
      li_pos := Pos ( '/', as_Link );
      while PosEx ( '/', as_Link, li_pos + 1 ) > 0 do
        li_pos := PosEx ( '/', as_Link, li_pos + 1 );
      ls_FileName:= copy ( as_link, li_pos+1, Length(as_link)-li_pos );
      ab_downloadReally := True;
      astl_temp1 := TStringList.Create ;
      try
        p_LoadStringList ( astl_temp1,  CST_DOWNLOAD_FILE+CST_EXTENSION_HTML );
        p_ReplaceLanguageString(astl_temp1,'Source',as_Link,[rfReplaceAll]);
        p_ReplaceLanguageString(astl_temp1,'SourceTitle',StringReplace(ls_FileName,'_',' ',[rfReplaceAll]),[rfReplaceAll]);
        if as_artist > ''
         Then p_ReplaceLanguageString(astl_temp1,'SourceArtist',' - '+as_artist,[rfReplaceAll])
         Else p_ReplaceLanguageString(astl_temp1,'SourceArtist','',[rfReplaceAll]);
        astl_downloadList.AddStrings(astl_temp1);

      finally
        astl_temp1.Destroy;
      end;
    end;

    const CST_documentwrite = 'document.write("';

    // Add a download to music download list
    procedure p_addText ( const as_filename : String ); overload;
    var lstl_Text : TStringList;
    Begin
      lstl_Text:=TStringList.Create;
      try
        lstl_Text.LoadFromFile(as_filename);
        lstl_Text.Text:=CST_documentwrite+StringReplace(fs_html_Lines(lstl_Text.Text,''),'"','\"',[rfReplaceAll])+'");';
        lstl_Text.SaveToFile(as_filename+CST_EXTENSION_JS);
        as_BackDirText:='';
      finally
        lstl_Text.Destroy;
      end;
    end;

    // Add a archive download to archive download list
    procedure p_addDownload ( const astl_downloadList : TStrings ; const as_Ext : String ; const ai_endext : Integer ); overload;
    var li_pos : Integer;
    Begin
      li_pos := PosEx ( as_Ext, ls_FileNameLower, ai_EndExt );
      if li_pos = 0 Then Exit;
      ab_downloadReally := True;
      p_LoadStringList ( astl_temp1,  CST_DOWNLOAD_FILE+CST_EXTENSION_HTML );
      p_MiniFileSource;
      p_replaceAllCase ( li_pos, as_Ext );
      if as_artist > ''
       Then p_ReplaceLanguageString(astl_temp1,'SourceArtist',' - '+as_artist,[rfReplaceAll])
       Else p_ReplaceLanguageString(astl_temp1,'SourceArtist','',[rfReplaceAll]);
      astl_downloadList.AddStrings(astl_temp1);
    end;

    procedure p_addDownload ( const astl_downloadList : TStrings ; const as_Ext : String ); overload;
    Begin
      p_addDownload ( astl_downloadList, as_Ext, li_endext );
    end;

    procedure p_addFile( const ab_add : Boolean; const as_Ext : String );
    var ls_FileWithoutExt     ,
        ls_FileNameWithoutExt : String;
        li_pos : Integer;

      procedure p_AddExt ( as_ext : String; const as_jplayerLabelFile : String );
      var ls_source64:String;
      Begin
        if ch_Names64.Checked
          Then ls_source64 := copy ( ls_FileNameWithoutExt, 1, 60 )
          Else ls_source64 := ls_FileNameWithoutExt;
      end;
    Begin

    end;

    function fs_MiniPath ( const as_path : String ) : String;
    Begin
      if ai_eraseBegin>length ( as_path )-5
       Then Result:='../'+ExtractFileName(as_path)
       Else Result := copy ( as_path, ai_eraseBegin, length ( as_path ) - ai_eraseBegin + 1 );
    end;

    function fb_ReplaceImgExt ( const as_image : String ; const as_extension : String ) : Boolean;
    Begin
      Result := FileExistsUTF8(as_image+as_extension);
      if Result
       Then
        Begin
         {$IFDEF WINDOWS}
         p_ReplaceLanguageString(astl_temp1,'SourcePoster',fs_RemplaceChar(fs_MiniPath (as_image)+as_extension,DirectorySeparator,'/'),[rfReplaceAll]);
         {$ELSE}
         p_ReplaceLanguageString(astl_temp1,'SourcePoster',fs_MiniPath (as_image)+as_extension,[rfReplaceAll]);
         {$ENDIF}
         Exit
        end;
      //WriteLn(fs_MiniPath (as_image)+as_extension+ ' '+as_image+as_extension);
    end;

   var lstl_DownloadInTextFile : TStringList;
       li_i : Integer;
begin
  astl_temp1.Clear ;
  lstl_temp2 := TStringListUTF8.Create;
  if ch_CSV.Checked Then
   lstl_DownloadInTextFile := TStringList.Create;
  try
   if fb_FindFiles ( lstl_temp2, as_Source, True, True, True, '*' ) Then
    Begin
     lstl_temp2.Sort;
     while lstl_temp2.count > 0 do
      Begin
        ls_FileSource := lstl_temp2.Strings [ 0 ];
        FindFirstUTF8( ls_FileSource,faanyfile,lsr_AttrSource);
        ls_FileName := lsr_AttrSource.Name ;
        FindCloseUTF8(lsr_AttrSource);
        if DirectoryExistsUTF8 ( ls_FileSource ) Then
          Begin
            p_addKeyWord(ls_FileName);
            if as_artist > ''
             Then ls_DirHtml := as_artist + ' - ' + ls_FileName
             Else ls_DirHtml := '';
            if ( ai_Level = 1 ) Then
              ab_foundAudio:=False;
            ls_BackDirText:=as_BackDirText+'../';
            lb_TextOneLanguage:=False;
        //    if ls_FileName='Bach'
           //  then ShowMessage(ls_BackDirText+' '+as_BackDirText);
            p_AddFiles ( ls_FileSource, ls_DirHtml, as_subdirForward+'../', astl_files, ai_eraseBegin, ai_Level + 1, astl_DirListAudio, astl_temp1, astl_HtmlDownloads, astl_downloadsafter, astl_Processes, ab_first, ab_foundAudio, ab_Root, ab_downloadReally, lb_TextOneLanguage, ls_BackDirText );
            if  ab_foundAudio
            and ( ai_Level = 1 ) // p_genHtmlHome will recall this recursive function
            and ch_RecurseSubDirs.Checked Then
             Begin
              p_genHtmlHome ( ls_FileSource+DirectorySeparator, as_subdirForward+'../', ls_FileName, ls_BackDirText, lb_TextOneLanguage, False );
              astl_DirListAudio.Add(ls_FileSource);
             end;
            if  ab_Root
            and ( ai_Level = 1 )
             Then
              p_IncProgress (pb_Progress);
          End
        Else
          if FileExistsUTF8 ( ls_FileSource ) Then
            Begin
               ls_FileNameLower := LowerCase(ls_FileName);
               li_EndExt := Length(ls_FileName)-4;
               p_addFile( ch_EPUB.Checked, CST_EXTENSION_EPUB );

               p_addFile( ch_PDF.Checked, CST_EXTENSION_PDF );
               p_addFile( ch_CSV.Checked, CST_EXTENSION_CSV );

{              if (pos ( CST_EXTENSION_TXT+CST_EXTENSION_JS  , ls_FileNameLower ) >= li_EndExt-3)
              and (pos ( ed_TXTName.Text+'.', ls_FileNameLower ) = 1 ) Then
                p_addOldText ( ls_FileSource );}
             if ch_ZIP.Checked
             and (pos ( CST_EXTENSION_ZIP, ls_FileNameLower ) >= li_EndExt) Then
               p_addDownload ( astl_HtmlDownloads, CST_EXTENSION_ZIP);
             if ch_Torrent.Checked
             and (pos ( CST_EXTENSION_TORRENT, ls_FileNameLower ) > 1) Then
               p_addDownload ( astl_Htmldownloads, CST_EXTENSION_TORRENT, 1);
             if ch_CSV.Checked
             and (pos ( CST_EXTENSION_CSV, ls_FileNameLower ) >= li_EndExt)
             and (pos ( CST_DOWNLOAD     , ls_FileNameLower ) = 1 ) Then
              Begin
                ab_downloadReally := True;
                p_LoadStrings(lstl_DownloadInTextFile, ls_FileSource, gs_WebBooks_CSVFiles_Are_Labels_with_links );
                for li_i := 0 to lstl_DownloadInTextFile.count -1 do
                 p_addExternalDownload ( astl_HtmlDownloads, lstl_DownloadInTextFile [ li_i ]);
              end;
             if ch_7ZIP.Checked
             and (pos ( CST_EXTENSION_7Z, ls_FileNameLower ) > li_EndExt) Then
               p_addDownload ( astl_HtmlDownloads, CST_EXTENSION_7Z);
             if ch_GZ.Checked
             and (pos ( CST_EXTENSION_GZ, ls_FileNameLower ) > li_EndExt) Then
               p_addDownload ( astl_HtmlDownloads, CST_EXTENSION_GZ);
            End;
        lstl_temp2.Delete(0);
      End ;
    end;
  finally
    if ch_CSV.Checked Then
      lstl_DownloadInTextFile.Destroy;
    lstl_temp2.Destroy;
  end;
End ;


procedure TF_WebBooks.fne_importChange(Sender: TObject);
var as_text : String;
Begin
  as_text := fne_import.{$IFDEF FPC}Filename{$ELSE}Text{$ENDIF};
  p_importAcceptFileName ( Self, CST_WebBooks, as_text, OnFormInfoIni);
End;
// procedure TF_WebBooks.fne_importAcceptFileName
// Importing ini on filename's accept
procedure TF_WebBooks.fne_importAcceptFileName(Sender: TObject;
  var Value: String);
var ls_FileImport : String;
begin
  ls_FileImport := {$IFDEF FPC}Value{$ELSE}fne_import.Text{$ENDIF};
  p_importAcceptFileName(Self,CST_WebBooks, ls_FileImport, OnFormInfoIni);
end;

procedure TF_WebBooks.p_Unfat(const as_directory : String);
var lstl_FilesToVerify : TStringListUTF8;
  ls_source64,ls_ext:String;
begin
  lstl_FilesToVerify := TStringListUTF8.Create;
  try
  finally
    lstl_FilesToVerify.Destroy;
  end;
end;

// procedure TF_WebBooks.bt_exportClick
// Export ini click event
procedure TF_WebBooks.bt_exportClick(Sender: TObject);
begin
  f_GetMainMemIniFile(nil,nil,nil,False,CST_WebBooks);
  p_exportClick(Self,fne_export.FileName,FileIniCopy,OnFormInfoIni);
end;


procedure TF_WebBooks.p_comment(const as_Comment: String);
begin
  p_Setcomments(lb_Comments,as_Comment,gs_Web_Generating );
end;

// procedure TF_WebBooks.p_genHtmlHome
// Default HTML page
procedure TF_WebBooks.p_genHtmlHome ( const as_directory, as_subdirForward, as_artist, as_backDirText : String;
                                       const ab_TextOneLanguage, ab_Root : Boolean );
var
  lstl_Temp1,
  lstl_HTMLHome,lstl_HTMLBody,lstl_HTMLDownloads,
  lstl_DirList,lstl_DirLine,lstl_ListDirAudio,
  lstl_downloadsAfter, lstl_processes : TStringListUTF8;
  ls_destination,ls_htmlpath: string;
  ls_Images,ls_BackDirText: string;
  li_Count, li_PathToDelete: integer;
  lb_first, lb_foundaudio, lb_TextOneLanguage, lb_downloadReally : boolean;
  procedure p_addRoot;
  var ls_IndexDir, ls_subdir : String ;
  Begin
    // load html file of button
    p_LoadStringList(lstl_HTMLDownloads,CST_INDEX_BUTTON+CST_EXTENSION_HTML);
    ls_IndexDir := copy ( as_directory, 1, Length(as_directory) - 1 );
    ls_subdir := '';
    repeat    // back to previous dir
      AppendStr ( ls_subdir, '../' );
      ls_IndexDir := ExtractSubDir ( ls_IndexDir );
    until (pos ( DirectorySeparator, ls_IndexDir ) = 0) or FileExistsUTF8(ls_IndexDir+DirectorySeparator+ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN);
    //create button
    p_ReplaceLanguageString(lstl_HTMLDownloads,'Link',ls_subdir+ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN,[rfReplaceAll]);
    p_ReplaceLanguageString(lstl_HTMLDownloads,'Caption',gs_Web_Back,[rfReplaceAll]);
  End;
begin
  p_comment (( gs_Web_Home )); // advert for user
  lstl_HTMLHome     := TStringListUTF8.Create;
  lstl_HTMLBody     := TStringListUTF8.Create;
  lstl_ListDirAudio := TStringListUTF8.Create;
  lstl_processes    := TStringListUTF8.Create;
  lstl_HTMLDownloads := TStringListUTF8.Create;
  lstl_Temp1         := TStringListUTF8.Create;
  p_LoadStringList ( lstl_HTMLBody, CST_INDEX_BUTTON+CST_EXTENSION_HTML, True );
  p_ReplaceLanguageString(lstl_HTMLBody,'Link',ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN,[rfReplaceAll]);
  p_ReplaceLanguageString(lstl_HTMLBody,'Caption',gs_Web_Back,[rfReplaceAll]);
  lstl_HTMLDownloads.AddStrings(lstl_HTMLBody);
  lstl_HTMLBody.clear;
  lb_downloadReally  := False;
  lb_TextOneLanguage := False;
  if ab_Root
  and ch_convert.Checked Then
   lstl_processes := TStringListUTF8.Create;
  // every found music can be downloaded
  if ch_downloads.Checked
    Then lstl_downloadsAfter := TStringListUTF8.Create;
  try
    p_ClearKeyWords;
    lb_first := True;
    li_PathToDelete := Length(as_directory)+1;
    try
      lb_foundaudio := False;
      ls_BackDirText:='';
      p_AddFiles ( as_directory, as_artist, as_subdirForward, lstl_HTMLHome, li_PathToDelete,1,lstl_ListDirAudio,lstl_Temp1,lstl_HTMLDownloads,lstl_downloadsAfter, lstl_processes, lb_first, lb_foundaudio, ab_Root, lb_downloadReally, lb_TextOneLanguage, ls_BackDirText );
    finally
      lstl_Temp1.Free;
    end;
    p_LoadStringList ( lstl_HTMLBody, CST_INDEX_BODY+CST_EXTENSION_HTML );
   if ch_downloads.Checked or lb_downloadReally Then
    Begin  // download and back link
      if ch_downloads.Checked
        Then lstl_HTMLDownloads.AddStrings(lstl_downloadsAfter);
      p_CreateAHtmlFile(lstl_HTMLDownloads, CST_DOWNLOAD, me_Description.Lines.Text,
         ExtractDirName(as_directory),
         as_artist + ' - ' + gs_Web_Downloads , gs_Web_Downloads, '', '');
      p_ReplaceLanguageString(lstl_HTMLDownloads,'SubDir',as_subdirForward,[rfReplaceAll]);
      p_saveFile(lstl_HTMLDownloads,gs_Web_Phase + gs_Web_Downloads,as_directory + ed_DownLoadName.Text + CST_EXTENSION_HTML_PLAIN);
      p_LoadStringList ( lstl_HTMLDownloads, CST_INDEX_BUTTON+CST_EXTENSION_HTML_PLAIN );
      p_ReplaceLanguageString(lstl_HTMLDownloads,'Link',ed_DownLoadName.Text+CST_EXTENSION_HTML_PLAIN,[rfReplaceAll]);
      p_ReplaceLanguageString(lstl_HTMLDownloads,'Caption',gs_Web_Downloads,[rfReplaceAll]);
      //adding download button to index
      p_ReplaceLanguageString(lstl_HTMLBody,'ButtonsToAdd',lstl_HTMLDownloads.Text+#10+'[ButtonsToAdd]',[rfReplaceAll]);
    End;
   //adding
   if as_subdirForward > '' Then
    Begin
     p_addRoot;
     p_ReplaceLanguageString(lstl_HTMLBody,'ButtonsToAdd',lstl_HTMLDownloads.Text+'[ButtonsToAdd]',[rfReplaceAll]);
    end;
    if lstl_ListDirAudio.Count>0 Then
      Begin
       lstl_DirList := TStringListUTF8.Create;
       lstl_DirLine := TStringListUTF8.Create;
        try
          while lstl_ListDirAudio.Count > 0  do
           Begin
            p_LoadStringList(lstl_DirLine,CST_INDEX_DIR+CST_EXTENSION_HTML);
            ls_destination:=lstl_ListDirAudio[0];
            ls_destination:=copy ( ls_destination, li_PathToDelete, Length(ls_destination)-li_PathToDelete+1 );
            {$IFDEF WINDOWS}
            p_ReplaceLanguageString(lstl_DirLine,'Directory','./'+StringReplace(StringReplace(ls_destination+DirectorySeparator+ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN,DirectorySeparator,'/',[rfReplaceAll]),'"','\"',[rfReplaceAll]),[rfReplaceAll]);
            {$ELSE}
            p_ReplaceLanguageString(lstl_DirLine,'Directory','./'+StringReplace(ls_destination+DirectorySeparator+ed_IndexName.Text+CST_EXTENSION_HTML_PLAIN,'"','\"',[rfReplaceAll]),[rfReplaceAll]);
            {$ENDIF}
            p_ReplaceLanguageString(lstl_DirLine,'Caption',StringReplace(ls_destination,DirectorySeparator,' - ',[rfReplaceAll]),[rfReplaceAll]);
            lstl_DirList.AddStrings(lstl_DirLine);
            lstl_ListDirAudio.Delete(0);
           end;

          p_ReplaceLanguageString(lstl_HTMLBody,'DirList','<ul>'+StringReplace(StringReplace(lstl_DirList.Text,'"','\"',[rfReplaceAll]),#10,'\n',[rfReplaceAll])+'</ul>',[rfReplaceAll]);
        finally
          lstl_DirList.Free;
          lstl_DirLine.Free;
        end;
        p_LoadStringList ( lstl_HTMLDownloads, CST_INDEX_BUTTON+CST_EXTENSION_HTML );
        p_ReplaceLanguageString(lstl_HTMLDownloads,'Link','#" onclick="DirectoriesShowHide();',[rfReplaceAll]);
        p_ReplaceLanguageString(lstl_HTMLDownloads,'Caption','...',[rfReplaceAll]);
        p_ReplaceLanguageString(lstl_HTMLBody,'ButtonsToAdd',lstl_HTMLDownloads.Text,[rfReplaceAll]);
      end
     else
      Begin
        p_ReplaceLanguageString(lstl_HTMLBody,'DirList','',[rfReplaceAll]);
        p_ReplaceLanguageString(lstl_HTMLBody,'ButtonsToAdd','',[rfReplaceAll]);
      end;
    if ch_CSV.checked
     Then
      Begin
       if as_directory=''
        Then ls_htmlpath:='/'
        else ls_htmlpath:=copy(as_directory,Length(AppendPathDelim(de_indexdir.Directory)),Length(as_directory));
      end
      else p_ReplaceLanguageString(lstl_HTMLBody,CST_DESCRIBE,fs_html_Lines(me_Description.Text ));
    lstl_HTMLHome.AddStrings(lstl_HTMLBody);
    lstl_HTMLBody.Clear;
    p_CreateAHtmlFile(lstl_HTMLHome, CST_INDEX, me_Description.Text,
      ExtractDirName(as_directory),
      as_artist , gs_Web_Home, '', '');
    p_ReplaceLanguageString(lstl_HTMLHome,'SubDir',as_subdirForward,[rfReplaceAll]);
    // saving the page
    p_saveFile(lstl_HTMLHome,gs_Web_Phase + ' - ' + gs_Web_Home,as_directory + ed_IndexName.Text + CST_EXTENSION_HTML_PLAIN);
    if ab_Root
    and ch_convert.Checked Then
      if (lstl_processes.Count > 0) Then
       Begin
        p_saveFile(lstl_processes,gs_Web_Phase + ' - ' + gs_Web_Convert,as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT);
        if MessageDlg(gs_Web_Convert,fs_RemplaceMsg(gs_Web_Script_will_be_executed,[as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT]),
                      mtConfirmation,mbYesNo,0) = mrYes Then
         Begin
          p_comment(fs_RemplaceMsg(gs_Web_Convert,[CST_EXTENSION_ODT]));
          {$IFDEF WINDOWS}
          fs_ExecuteProcess('"'+StringReplace(as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT,'"','\"',[rfReplaceAll])+'"','',False)
          {$ELSE}
          fs_ExecuteProcess('sh','"'+StringReplace(as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT,'"','\"',[rfReplaceAll])+'"',False)
          {$ENDIF}
         End;
       end
       else if FileExistsUTF8(as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT) then
       if MessageDlg(gs_Web_Delete_File,fs_RemplaceMsg(gs_Web_Delete_File_confirm,[as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT]),mtConfirmation,mbYesNo,0) = mrYes
        Then
         DeleteFileUTF8(as_directory + CST_SCRIPT_FILE + CST_EXTENSION_SCRIPT);
  finally
    lstl_HTMLHome    .Destroy;
    lstl_HTMLBody    .Destroy;
    lstl_ListDirAudio.Destroy;
    if ch_downloads.Checked
      Then lstl_downloadsAfter.Destroy;
    lstl_HTMLDownloads.Destroy;
    if ab_Root
    and ch_convert.Checked Then
     lstl_processes.Destroy;
  end;
end;


// procedure TF_WebBooks.p_CreateAHtmlFile
// Creating a HTML page from parameters
procedure TF_WebBooks.p_CreateAHtmlFile(const astl_Destination: TStrings;
  const as_BeginingFile,
  as_Describe, as_file, as_Title, as_LittleTitle, as_LongTitle: string;
  const as_Subdir: string = '';
  const as_ExtFile: string =
  CST_EXTENSION_HTML_PLAIN;
  const as_BeforeHTML: string = ''; const astl_Body : TStrings = nil );
var ls_title : String;
begin
  if not assigned ( gstl_HeadKeyWords ) Then
    Abort;       // can quit while creating
  if as_Title = ''
   Then ls_title := 'Lazarus Web Books'
   else ls_title := 'Lazarus Web Books - '+as_Title;
  p_CreateHTMLFile(nil, astl_Destination, '',
     as_Describe,
     gstl_HeadKeyWords.Text, ls_Title,
    as_LongTitle, as_BeginingFile + '1' + as_ExtFile, as_BeginingFile + '2' +
    as_ExtFile, as_BeginingFile + '3' + as_ExtFile, as_BeginingFile +
    '4' + as_ExtFile, as_Subdir, as_BeforeHTML, gs_Web_Language, astl_Body, False );
end;

// procedure TF_WebBooks.DoAfterInit
// initing components and ini
procedure TF_WebBooks.DoAfterInit( const ab_Ini : Boolean = True );
begin
  {$IFNDEF FPC}
  de_indexdir.InitialDir:=GetWindir(CSIDL_DESKTOPDIRECTORY);
  {$ENDIF}

  // Reading ini
  if not ab_Ini Then
    Exit;
  f_GetMainMemIniFile(nil,nil,nil,False,CST_WebBooks);
  OnFormInfoIni.p_ExecuteLecture(Self);
end;

// procedure TF_WebBooks.FormCreate
// initializing software and form
constructor TF_WebBooks.Create(aowner:TComponent);
var ls_Path : String ;
Begin
  inherited;
  gs_Root:=ExtractFileDir(Application.ExeName)+DirectorySeparator;
  AppendStr(gs_Root,'WebBooks'+DirectorySeparator);//pas dans plugins pour l'exe

  ls_Path:=GetAppConfigDir(False) + DirectorySeparator + CST_SUBDIR_THEMES;
  try
    cb_Themes.Items.Clear;
    fb_FindFiles(cb_Themes.Items, gs_Root + CST_SUBDIR_THEMES, False);
    if not DirectoryExistsUTF8(ls_Path)
     Then
      fb_CreateDirectoryStructure(ls_Path);
    fb_FindFiles(cb_Themes.Items, ls_Path, False);

  except

  end;

  try
    cb_Files.Items.Clear;
    fb_FindFiles(cb_Files.Items, gs_Root + CST_SUBDIR_FILES, False);
  except

  end;

  // ici mettre la taille initiale car avec les skins, les fenetres se resize
  Width := 640;
  Height := 400;
  Caption := fs_getCorrectString(CST_WebBooks_WithLicense+' : '+gs_Web_FORM_CAPTION);
  OnFormInfoIni.p_ExecuteLecture(Self);
end;



{$IFNDEF FPC}
initialization
  p_ConcatVersion ( gVer_WebPlayer );
{$ENDIF}


end.

